import unittest
from datetime import datetime

from securenative.utils.date_utils import DateUtils


class DateUtilTest(unittest.TestCase):

    def test_to_timestamp(self):
        iso_8601_date = "2020-05-20T15:07:13Z"
        result = DateUtils.to_timestamp(iso_8601_date)

        self.assertEqual(datetime.strptime(iso_8601_date, '%Y-%m-%dT%H:%M:%S.%f'[:-3] + 'Z'), result)




from enum import Enum


class EventTypes(Enum):
    LOG_IN = "sn.user.login"
    LOG_IN_CHALLENGE = "sn.user.login.challenge"
    LOG_IN_FAILURE = "sn.user.login.failure"
    LOG_OUT = "sn.user.logout"
    SIGN_UP = "sn.user.signup"
    AUTH_CHALLENGE = "sn.user.auth.challenge"
    AUTH_CHALLENGE_SUCCESS = "sn.user.auth.challenge.success"
    AUTH_CHALLENGE_FAILURE = "sn.user.auth.challenge.failure"
    TWO_FACTOR_DISABLE = "sn.user.2fa.disable"
    EMAIL_UPDATE = "sn.user.email.update"
    PASSWORD_REST = "sn.user.password.reset"
    PASSWORD_REST_SUCCESS = "sn.user.password.reset.success"
    PASSWORD_UPDATE = "sn.user.password.update"
    PASSWORD_REST_FAILURE = "sn.user.password.reset.failure"
    USER_INVITE = "sn.user.invite"
    ROLE_UPDATE = "sn.user.role.update"
    PROFILE_UPDATE = "sn.user.profile.update"
    PAGE_VIEW = "sn.user.page.view"
    VERIFY = "sn.verify"



import ipaddress
import re
import socket


class IpUtils(object):
    VALID_IPV4_PATTERN = re.compile("(([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.){3}([01]?\\d\\d?|2[0-4]\\d|25[0-5])")
    VALID_IPV6_PATTERN = re.compile("([0-9a-f]{1,4}:){7}([0-9a-f]){1,4}")

    @staticmethod
    def is_ip_address(ip_address):
        if IpUtils.VALID_IPV4_PATTERN.match(ip_address):
            return True
        if IpUtils.VALID_IPV6_PATTERN.match(ip_address):
            return True
        return False

    @staticmethod
    def is_valid_public_ip(ip_address):
        try:
            socket.inet_aton(ip_address)
        except socket.error:
            return False

        ip = ipaddress.IPv4Address(ip_address)
        if ip.is_loopback \
                or not ip.is_global \
                or ip.is_private \
                or ip.is_link_local \
                or ip.is_multicast \
                or ip.is_reserved \
                or ip.is_unspecified:
            return False
        return True

    @staticmethod
    def is_loop_back(ip_address):
        return ipaddress.IPv4Address(ip_address).is_loopback



from securenative.config.securenative_options import SecureNativeOptions
from securenative.enums.failover_strategy import FailOverStrategy


class ConfigurationBuilder(object):

    def __init__(self):
        self.api_key = None
        self.api_url = "https://api.securenative.com/collector/api/v1"
        self.interval = 1000
        self.max_events = 1000
        self.timeout = 1500
        self.auto_send = True
        self.disable = False
        self.log_level = "CRITICAL"
        self.fail_over_strategy = FailOverStrategy.FAIL_OPEN.value

    @staticmethod
    def default_config_builder():
        return ConfigurationBuilder()

    def with_api_key(self, api_key):
        self.api_key = api_key
        return self

    def with_api_url(self, api_url):
        self.api_url = api_url
        return self

    def with_interval(self, interval):
        self.interval = interval
        return self

    def with_max_events(self, max_events):
        self.max_events = max_events
        return self

    def with_timeout(self, timeout):
        self.timeout = timeout
        return self

    def with_auto_send(self, auto_send):
        self.auto_send = auto_send
        return self

    def with_disable(self, disable):
        self.disable = disable
        return self

    def with_log_level(self, log_level):
        self.log_level = log_level
        return self

    def with_fail_over_strategy(self, fail_over_strategy):
        if fail_over_strategy != FailOverStrategy.FAIL_OPEN.value and \
                fail_over_strategy != FailOverStrategy.FAIL_CLOSED.value:
            self.fail_over_strategy = FailOverStrategy.FAIL_OPEN.value
            return self
        self.fail_over_strategy = fail_over_strategy
        return self

    @staticmethod
    def get_default_securenative_options():
        return SecureNativeOptions()



from securenative.exceptions.securenative_invalid_options_exception import SecureNativeInvalidOptionsException
from securenative.models.event_options import EventOptions
from securenative.models.user_traits import UserTraits


class EventOptionsBuilder(object):
    MAX_PROPERTIES_SIZE = 10

    def __init__(self, event_type):
        self.event_options = EventOptions(event_type)

    def with_user_id(self, user_id):
        self.event_options.user_id = user_id
        return self

    def with_user_traits(self, user_traits):
        self.event_options.user_traits = user_traits
        return self

    def with_user(self, name, email, created_at=None):
        self.event_options.user_traits = UserTraits(name, email, created_at)
        return self

    def with_context(self, context):
        self.event_options.context = context
        return self

    def with_properties(self, properties):
        self.event_options.properties = properties
        return self

    def with_timestamp(self, timestamp):
        self.event_options.timestamp = timestamp
        return self

    def build(self):
        if self.event_options.properties is not None \
                and len(self.event_options.properties) > self.MAX_PROPERTIES_SIZE:
            raise SecureNativeInvalidOptionsException(
                "You can have only up to {} custom properties", self.MAX_PROPERTIES_SIZE)
        return self.event_options



from datetime import datetime


class DateUtils(object):

    @staticmethod
    def to_timestamp(date):
        if not date:
            return datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'
        return datetime.strptime(date, '%Y-%m-%dT%H:%M:%S.%f'[:-3] + 'Z')



class EventOptions(object):

    def __init__(self, event, user_id=None, user_traits=None,
                 context=None, properties=None, timestamp=None):
        self.event = event
        self.user_id = user_id
        self.user_traits = user_traits
        self.context = context
        self.properties = properties
        self.timestamp = timestamp



import json
import unittest
import responses

from securenative.api_manager import ApiManager
from securenative.config.configuration_manager import ConfigurationManager
from securenative.context.context_builder import ContextBuilder
from securenative.enums.event_types import EventTypes
from securenative.enums.risk_level import RiskLevel
from securenative.event_manager import EventManager
from securenative.event_options_builder import EventOptionsBuilder
from securenative.exceptions.securenative_invalid_options_exception import SecureNativeInvalidOptionsException
from securenative.models.user_traits import UserTraits
from securenative.models.verify_result import VerifyResult


class ApiManagerTest(unittest.TestCase):

    def setUp(self):
        self.context = ContextBuilder(). \
            with_ip("127.0.0.1"). \
            with_client_token("SECURED_CLIENT_TOKEN"). \
            with_headers(
            {
                "user-agent": "Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405"
            }).build()

        self.event_options = EventOptionsBuilder(EventTypes.LOG_IN). \
            with_user_id("USER_ID"). \
            with_user_traits(UserTraits("USER_NAME", "USER_EMAIL")). \
            with_context(self.context). \
            with_properties({"prop1": "CUSTOM_PARAM_VALUE",
                             "prop2": True,
                             "prop3": 3}).build()

    @responses.activate
    def test_track_event(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY"). \
            with_auto_send(True). \
            with_interval(10). \
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        expected = "{\"eventType\":\"sn.user.login\",\"userId\":\"USER_ID\",\"userTraits\":{" \
                   "\"name\":\"USER_NAME\",\"email\":\"USER_EMAIL\",\"createdAt\":null},\"request\":{" \
                   "\"cid\":null,\"vid\":null,\"fp\":null,\"ip\":\"127.0.0.1\",\"remoteIp\":null,\"headers\":{" \
                   "\"user-agent\":\"Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) " \
                   "AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405\"},\"url\":null,\"method\":null}," \
                   "\"properties\":{\"prop2\":true,\"prop1\":\"CUSTOM_PARAM_VALUE\",\"prop3\":3}}"

        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/track",
                      json=json.loads(expected), status=200)
        event_manager = EventManager(options)
        event_manager.start_event_persist()
        api_manager = ApiManager(event_manager, options)

        try:
            api_manager.track(self.event_options)
        finally:
            event_manager.stop_event_persist()

    @responses.activate
    def test_securenative_invalid_options_exception(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY"). \
            with_auto_send(True). \
            with_interval(10). \
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        properties = {}
        for i in range(1, 12):
            properties[i] = i

        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/track",
                      json={}, status=200)
        event_manager = EventManager(options)
        event_manager.start_event_persist()
        api_manager = ApiManager(event_manager, options)

        try:
            with self.assertRaises(SecureNativeInvalidOptionsException):
                api_manager.track(EventOptionsBuilder(
                    EventTypes.LOG_IN).with_properties(properties).build())
        finally:
            event_manager.stop_event_persist()

    @responses.activate
    def test_verify_event(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY"). \
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/verify",
                      json={
                          "riskLevel": "medium",
                          "score": 0.32,
                          "triggers": [
                              "New IP",
                              "New City"
                          ]
                      }, status=200)
        verify_result = VerifyResult(RiskLevel.LOW, 0, None)

        event_manager = EventManager(options)
        event_manager.start_event_persist()
        api_manager = ApiManager(event_manager, options)

        result = api_manager.verify(self.event_options)

        self.assertIsNotNone(result)
        self.assertEqual(result.risk_level, verify_result.risk_level.value)
        self.assertEqual(result.score, verify_result.score)
        self.assertEqual(result.triggers, verify_result.triggers)


if __name__ == '__main__':
    unittest.main()



from securenative.api_manager import ApiManager
from securenative.config.configuration_builder import ConfigurationBuilder
from securenative.config.configuration_manager import ConfigurationManager
from securenative.context.context_builder import ContextBuilder
from securenative.event_manager import EventManager
from securenative.exceptions.securenative_config_exception import SecureNativeConfigException
from securenative.exceptions.securenative_sdk_Illegal_state_exception import SecureNativeSDKIllegalStateException
from securenative.exceptions.securenative_sdk_exception import SecureNativeSDKException
from securenative.logger import Logger
from securenative.utils.signature_utils import SignatureUtils
from securenative.utils.utils import Utils


class SecureNative:
    _securenative = None

    def __init__(self, options):
        if Utils.is_null_or_empty(options.api_key):
            raise SecureNativeSDKException("You must pass your SecureNative api key")

        self._options = options
        self._event_manager = EventManager(self._options)

        if self._options.api_url:
            self._event_manager.start_event_persist()

        self._api_manager = ApiManager(self._event_manager, self._options)
        Logger.init_logger(self._options.log_level)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._event_manager.stop_event_persist()

    @classmethod
    def init_with_options(cls, options):
        if cls._securenative is None:
            cls._securenative = SecureNative(options)
            return cls._securenative
        else:
            Logger.debug('This SDK was already initialized.')
            raise SecureNativeSDKException(u'This SDK was already initialized.')

    @classmethod
    def init_with_api_key(cls, api_key):
        if Utils.is_null_or_empty(api_key):
            raise SecureNativeConfigException("You must pass your SecureNative api key")

        if cls._securenative is None:
            builder = ConfigurationBuilder().default_config_builder()
            options = builder.with_api_key(api_key)
            cls._securenative = SecureNative(options)
            return cls._securenative
        else:
            Logger.debug('This SDK was already initialized.')
            raise SecureNativeSDKException(u'This SDK was already initialized.')

    @classmethod
    def init(cls):
        options = ConfigurationManager.load_config()
        return cls.init_with_options(options)

    @classmethod
    def get_instance(cls):
        if not cls._securenative:
            raise SecureNativeSDKIllegalStateException()
        return cls._securenative

    def get_options(self):
        return self._options

    @staticmethod
    def config_builder():
        return ConfigurationBuilder.default_config_builder()

    @staticmethod
    def context_builder():
        return ContextBuilder.default_context_builder()

    def track(self, event_options):
        return self._api_manager.track(event_options)

    def verify(self, event_options):
        return self._api_manager.verify(event_options)

    @classmethod
    def _flush(cls):
        cls._securenative = None

    def verify_request_payload(self, request):
        request_signature = request.header[SignatureUtils.SignatureHeader]
        body = request.body

        SignatureUtils.is_valid_signature(self._options.api_key, body, request_signature)



import os


class VersionUtils(object):

    @staticmethod
    def get_version():
        root_dir = os.path.dirname(os.path.abspath(__file__)).replace("/securenative/utils", "")
        path = os.path.join(root_dir, "VERSION")
        with open(path) as f:
            return f.read()



class SecureNativeContext(object):

    def __init__(self, client_token=None, ip=None, remote_ip=None, headers=None, url=None, method=None, body=None):
        self.client_token = client_token
        self.ip = ip
        self.remote_ip = remote_ip
        self.headers = headers
        self.url = url
        self.method = method
        self.body = body



import hmac
import hashlib


class SignatureUtils(object):
    SignatureHeader = "x-securenative"

    @staticmethod
    def is_valid_signature(api_key, payload, header_signature):
        try:
            key = api_key.encode('utf-8')
            body = payload.encode('utf-8')
            comparison_signature = hmac.new(key, body, hashlib.sha512).hexdigest()
            return hmac.compare_digest(comparison_signature, header_signature)
        except Exception:
            return False



from securenative.context.securenative_context import SecureNativeContext
from securenative.utils.request_utils import RequestUtils
from securenative.utils.utils import Utils


class ContextBuilder(object):

    def __init__(self):
        self.context = SecureNativeContext()

    def with_client_token(self, client_token):
        self.context.client_token = client_token
        return self

    def with_ip(self, ip):
        self.context.ip = ip
        return self

    def with_remote_ip(self, remote_ip):
        self.context.remote_ip = remote_ip
        return self

    def with_headers(self, headers):
        self.context.headers = headers
        return self

    def with_url(self, url):
        self.context.url = url
        return self

    def with_method(self, method):
        self.context.method = method
        return self

    def with_body(self, body):
        self.context.body = body
        return self

    @staticmethod
    def default_context_builder():
        return ContextBuilder()

    @staticmethod
    def from_http_request(request):
        try:
            client_token = request.cookies[RequestUtils.SECURENATIVE_COOKIE]
        except AttributeError:
            client_token = None

        try:
            headers = request.headers
        except AttributeError:
            headers = None

        if Utils.is_null_or_empty(client_token):
            client_token = RequestUtils.get_secure_header_from_request(headers)

        return ContextBuilder()\
            .with_url(request.url)\
            .with_method(request.method)\
            .with_headers(headers)\
            .with_client_token(client_token)\
            .with_ip(RequestUtils.get_client_ip_from_request(request))\
            .with_remote_ip(RequestUtils.get_remote_ip_from_request(request))\
            .with_body(None)

    def build(self):
        return self.context



import logging


class Logger(object):
    logger = logging.getLogger()

    @staticmethod
    def init_logger(level):
        logging.basicConfig(level=level, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s")

    @staticmethod
    def info(msg):
        Logger.logger.info(msg)

    @staticmethod
    def debug(msg):
        Logger.logger.debug(msg)

    @staticmethod
    def warning(msg):
        Logger.logger.warning(msg)

    @staticmethod
    def error(msg):
        Logger.logger.error(msg)



import json

from securenative.enums.api_route import ApiRoute
from securenative.enums.failover_strategy import FailOverStrategy
from securenative.enums.risk_level import RiskLevel
from securenative.logger import Logger
from securenative.models.sdk_event import SDKEvent
from securenative.models.verify_result import VerifyResult


class ApiManager(object):

    def __init__(self, event_manager, securenative_options):
        self.event_manager = event_manager
        self.options = securenative_options

    def track(self, event_options):
        Logger.debug("Track event call")
        event = SDKEvent(event_options, self.options)
        return self.event_manager.send_async(event, ApiRoute.TRACK.value)

    def verify(self, event_options):
        Logger.debug("Verify event call")
        event = SDKEvent(event_options, self.options)
        try:
            res = json.loads(self.event_manager.send_sync(event, ApiRoute.VERIFY.value, False))
            return VerifyResult(res["riskLevel"], res["score"], res["triggers"])
        except Exception as e:
            Logger.debug("Failed to call verify; {}".format(e))
            if self.options.fail_over_strategy is FailOverStrategy.FAIL_OPEN.value:
                return VerifyResult(RiskLevel.LOW.value, 0, None)
            return VerifyResult(RiskLevel.HIGH.value, 1, None)



class RequestContext(object):

    def __init__(self, cid=None, vid=None, fp=None, ip=None,
                 remote_ip=None, headers=None, url=None, method=None):
        self.cid = cid
        self.vid = vid
        self.fp = fp
        self.ip = ip
        self.remote_ip = remote_ip
        self.headers = headers
        self.url = url
        self.method = method


class RequestContextBuilder(object):

    def __init__(self, cid=None, vid=None, fp=None, ip=None,
                 remote_ip=None, headers=None, url=None, method=None):
        self.cid = cid
        self.vid = vid
        self.fp = fp
        self.ip = ip
        self.remote_ip = remote_ip
        self.headers = headers
        self.url = url
        self.method = method

    def with_cid(self, cid):
        self.cid = cid
        return self

    def with_vid(self, vid):
        self.vid = vid
        return self

    def with_fp(self, fp):
        self.fp = fp
        return self

    def with_ip(self, ip):
        self.ip = ip
        return self

    def with_remote_ip(self, remote_ip):
        self.remote_ip = remote_ip
        return self

    def with_headers(self, headers):
        self.headers = headers
        return self

    def with_url(self, url):
        self.url = url
        return self

    def with_method(self, method):
        self.method = method
        return self

    def build(self):
        return RequestContext(self.cid, self.vid, self.fp, self.ip, self.remote_ip, self.headers, self.url, self.method)



import json
import threading
import time

from securenative.config.securenative_options import SecureNativeOptions
from securenative.exceptions.securenative_http_exception import SecureNativeHttpException
from securenative.http.securenative_http_client import SecureNativeHttpClient
from securenative.logger import Logger


class QueueItem:
    def __init__(self, url, body, retry):
        self.url = url
        self.body = body
        self.retry = retry


class EventManager:
    def __init__(self, options=SecureNativeOptions(), http_client=None):
        if options.api_key is None:
            raise ValueError('API key cannot be None, please get your API key from SecureNative console.')

        if not http_client:
            self.http_client = SecureNativeHttpClient(options)
        else:
            self.http_client = http_client

        self.queue = list()
        self.thread = threading.Thread(target=self.run, daemon=True)
        self.thread.start()

        self.options = options
        self.send_enabled = False
        self.attempt = 0
        self.coefficients = [1, 1, 2, 3, 5, 8, 13]
        self.thread = None
        self.interval = options.interval

    def send_async(self, event, resource_path):
        if self.options.disable:
            Logger.warning("SDK is disabled. no operation will be performed")
            return

        item = QueueItem(
            resource_path,
            json.dumps(EventManager.serialize(event)),
            False
        )

        self.queue.append(item)
        if self._is_queue_full():
            self.queue = self.queue[:len(self.queue - 1)]

    def flush(self):
        for item in self.queue:
            self.http_client.post(item.url, item.body)

    def send_sync(self, event, resource_path, retry):
        if self.options.disable:
            Logger.warning("SDK is disabled. no operation will be performed")
            return

        Logger.debug("Attempting to send event {}".format(event))
        res = self.http_client.post(
            resource_path,
            json.dumps(EventManager.serialize(event))
        )
        if res.status_code != 200:
            Logger.info("SecureNative failed to call endpoint {} with event {}. adding back to queue".format(
                resource_path, event))
            item = QueueItem(
                resource_path,
                json.dumps(EventManager.serialize(event)),
                retry
            )
            self.queue.append(item)
            if self._is_queue_full():
                self.queue = self.queue[:len(self.queue - 1)]
        return res

    def _is_queue_full(self):
        return len(self.queue) > self.options.max_events

    def run(self):
        while True:
            if len(self.queue) > 0 and self.send_enabled:
                for item in self.queue:
                    try:
                        res = self.http_client.post(item.url, item.body)
                        if res.status_code == 401:
                            item.retry = False
                        elif res.status_code != 200:
                            item.retry = True

                        Logger.debug("Event successfully sent; {}".format(item.body))
                    except Exception as e:
                        Logger.error("Failed to send event; {}".format(e))
                        if item.retry:
                            if len(self.coefficients) == self.attempt + 1:
                                self.attempt = 0

                            back_off = self.coefficients[self.attempt] * self.options.interval
                            Logger.debug("Automatic back-off of {}".format(back_off))
                            self.send_enabled = False
                            time.sleep(back_off)
                            self.send_enabled = True
                time.sleep(self.interval/1000)

    def start_event_persist(self):
        Logger.debug("Starting automatic event persistence")
        if self.options.auto_send or self.send_enabled:
            self.send_enabled = True
        else:
            Logger.debug("Automatic event persistence is disabled, you should persist events manually")

    def stop_event_persist(self):
        if self.send_enabled:
            Logger.debug("Attempting to stop automatic event persistence")
            try:
                self.flush()
                if self.thread:
                    self.thread.stop()
            except ValueError as e:
                Logger.error("Could not stop event scheduler; {}".format(e))

            Logger.debug("Stopped event persistence")

    @staticmethod
    def serialize(obj):
        return {
            "rid": obj.rid,
            "eventType": obj.event_type,
            "userId": obj.user_id,
            "userTraits": {
                "name": obj.user_traits.name,
                "email": obj.user_traits.email,
                "createdAt": obj.user_traits.created_at,
            },
            "request": {
                "cid": obj.request.cid,
                "vid": obj.request.vid,
                "fp": obj.request.fp,
                "ip": obj.request.ip,
                "remoteIp": obj.request.remote_ip,
                "method": obj.request.method,
                "url": obj.request.url,
                "headers": obj.request.headers
            },
            "timestamp": obj.timestamp,
            "properties": obj.properties,
        }



import configparser
import os
import unittest

from securenative.config.configuration_manager import ConfigurationManager
from securenative.enums.failover_strategy import FailOverStrategy


class ConfigurationManagerTest(unittest.TestCase):

    def setUp(self):
        self.config_file_path = "/tmp/securenative.ini"

    def create_ini_file(self, conf):
        os.environ["SECURENATIVE_COMFIG_FILE"] = self.config_file_path

        try:
            os.remove(self.config_file_path)
        except FileNotFoundError:
            pass

        config = configparser.ConfigParser()

        for key, value in conf.items():
            config.set("DEFAULT", key.upper(), value)

        with open(self.config_file_path, "w") as configfile:
            config.write(configfile)

    def test_parse_config_file_correctly(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        config = {
            "SECURENATIVE_API_KEY": "SOME_API_KEY",
            "SECURENATIVE_APP_NAME": "SOME_APP_NAME",
            "SECURENATIVE_API_URL": "SOME_API_URL",
            "SECURENATIVE_INTERVAL": "1000",
            "SECURENATIVE_HEARTBEAT_INTERVAL": "5000",
            "SECURENATIVE_MAX_EVENTS": "100",
            "SECURENATIVE_TIMEOUT": "1500",
            "SECURENATIVE_AUTO_SEND": "True",
            "SECURENATIVE_DISABLE": "False",
            "SECURENATIVE_LOG_LEVEL": "Critical",
            "SECURENATIVE_FAILOVER_STRATEGY": "fail-closed"
        }

        self.create_ini_file(config)
        options = ConfigurationManager.load_config()

        self.assertIsNotNone(options)
        self.assertEqual(options.api_key, "SOME_API_KEY")
        self.assertEqual(options.api_url, "SOME_API_URL")
        self.assertEqual(options.auto_send, "True")
        self.assertEqual(options.disable, "False")
        self.assertEqual(options.fail_over_strategy, FailOverStrategy.FAIL_CLOSED.value)
        self.assertEqual(options.interval, "1000")
        self.assertEqual(options.log_level, "Critical")
        self.assertEqual(options.max_events, "100")
        self.assertEqual(options.timeout, "1500")

    def test_ignore_unknown_config_in_properties_file(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        config = {
            "SECURENATIVE_TIMEOUT": "1500",
            "SECURENATIVE_UNKNOWN_KEY": "SOME_UNKNOWN_KEY"
        }

        self.create_ini_file(config)
        options = ConfigurationManager.load_config()

        self.assertIsNotNone(options)
        self.assertEqual(options.timeout, "1500")

    def test_handle_invalid_config_file(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        config = {"bla": "bla"}

        self.create_ini_file(config)
        options = ConfigurationManager.load_config()

        self.assertIsNotNone(options)

    def ignore_invalid_config_file_entries(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        config = {
            "SECURENATIVE_API_KEY": 1,
            "SECURENATIVE_API_URL": 3,
            "SECURENATIVE_TIMEOUT": "bad timeout",
            "SECURENATIVE_FAILOVER_STRATEGY": "fail-what"
        }

        self.create_ini_file(config)
        options = ConfigurationManager.load_config()

        self.assertIsNotNone(options)
        self.assertEqual(options.fail_over_strategy, FailOverStrategy.FAIL_OPEN)

    def test_load_default_config(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        options = ConfigurationManager.load_config()

        self.assertIsNotNone(options)
        self.assertIsNone(options.api_key)
        self.assertEqual(options.api_url, "https://api.securenative.com/collector/api/v1")
        self.assertEqual(str(options.interval), "1000")
        self.assertEqual(options.timeout, "1500")
        self.assertEqual(str(options.max_events), "1000")
        self.assertEqual(str(options.auto_send), "True")
        self.assertEqual(str(options.disable), "False")
        self.assertEqual(options.log_level, "CRITICAL")
        self.assertEqual(options.fail_over_strategy, FailOverStrategy.FAIL_OPEN.value)

    def test_get_config_from_env_variables(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        os.environ["SECURENATIVE_API_KEY"] = "SOME_ENV_API_KEY"
        os.environ["SECURENATIVE_API_URL"] = "SOME_API_URL"
        os.environ["SECURENATIVE_INTERVAL"] = "6000"
        os.environ["SECURENATIVE_MAX_EVENTS"] = "700"
        os.environ["SECURENATIVE_TIMEOUT"] = "1700"
        os.environ["SECURENATIVE_AUTO_SEND"] = "False"
        os.environ["SECURENATIVE_DISABLE"] = "True"
        os.environ["SECURENATIVE_LOG_LEVEL"] = "Debug"
        os.environ["SECURENATIVE_FAILOVER_STRATEGY"] = "fail-closed"

        options = ConfigurationManager.load_config()

        self.assertEqual(options.api_key, "SOME_ENV_API_KEY")
        self.assertEqual(options.api_url, "SOME_API_URL")
        self.assertEqual(options.interval, "6000")
        self.assertEqual(options.timeout, "1700")
        self.assertEqual(options.max_events, "700")
        self.assertEqual(options.auto_send, "False")
        self.assertEqual(options.disable, "True")
        self.assertEqual(options.log_level, "Debug")
        self.assertEqual(options.fail_over_strategy, FailOverStrategy.FAIL_CLOSED.value)

    def test_default_values_for_invalid_enum_config_props(self):
        try:
            os.remove(self.config_file_path)
            del os.environ["SECURENATIVE_API_KEY"]
            del os.environ["SECURENATIVE_API_URL"]
            del os.environ["SECURENATIVE_INTERVAL"]
            del os.environ["SECURENATIVE_MAX_EVENTS"]
            del os.environ["SECURENATIVE_TIMEOUT"]
            del os.environ["SECURENATIVE_AUTO_SEND"]
            del os.environ["SECURENATIVE_DISABLE"]
            del os.environ["SECURENATIVE_LOG_LEVEL"]
            del os.environ["SECURENATIVE_FAILOVER_STRATEGY"]
        except FileNotFoundError:
            pass
        except KeyError:
            pass

        config = {
            "SECURENATIVE_FAILOVER_STRATEGY": "fail-something"
        }

        self.create_ini_file(config)
        options = ConfigurationManager.load_config()

        self.assertIsNotNone(options)
        self.assertEqual(options.fail_over_strategy, FailOverStrategy.FAIL_OPEN.value)



import unittest

from securenative.utils.ip_utils import IpUtils


class IpUtilTest(unittest.TestCase):

    def test_is_ip_address_valid_ipv4(self):
        valid_ipv4 = "172.16.254.1"
        self.assertTrue(IpUtils.is_ip_address(valid_ipv4))

    def test_is_ip_address_valid_ipv6(self):
        valid_ipv6 = "2001:db8:1234:0000:0000:0000:0000:0000"
        self.assertTrue(IpUtils.is_ip_address(valid_ipv6))

    def test_is_ip_address_invalid_ipv4(self):
        invalid_ipv4 = "172.16.2541"
        self.assertFalse(IpUtils.is_ip_address(invalid_ipv4))

    def test_is_ip_address_invalid_ipv6(self):
        invalid_ipv6 = "2001:db8:1234:0000"
        self.assertFalse(IpUtils.is_ip_address(invalid_ipv6))

    def test_is_valid_public_ip(self):
        ip = "64.71.222.37"
        self.assertTrue(IpUtils.is_valid_public_ip(ip))

    def test_is_not_valid_public_ip(self):
        ip = "10.0.0.0"
        self.assertFalse(IpUtils.is_valid_public_ip(ip))

    def test_is_valid_loopback_ip(self):
        ip = "127.0.0.1"
        self.assertTrue(IpUtils.is_loop_back(ip))



import unittest

from securenative.utils.signature_utils import SignatureUtils


class SignatureUtilTest(unittest.TestCase):

    def test_verify_request_payload(self):
        signature = "c4574c1748064735513697750c6223ff36b03ae3b85b160ce8788557d01e1d9d1c9cd942074323ee0061d3dcc8c94359c5acfa6eee8e2da095b3967b1a88ab73"
        payload = "{\"id\":\"4a9157ffbd18cfbd73a57298\",\"type\":\"security-action\",\"flow\":{\"id\":\"62298c73a9bb433fbd1f75984a9157fd\",\"name\":\"Block user that violates geo velocity\"},\"userId\":\"73a9bb433fbd1f75984a9157\",\"userTraits\":{\"name\":\"John Doe\",\"email\":\"john.doe@gmail.com\"},\"request\":{\"ip\":\"10.0.0.0\",\"fp\":\"9bb433fb984a9157d1f7598\"},\"action\":\"block\",\"properties\":{\"type\":\"customer\"},\"timestamp\":\"2020-02-23T22:28:55.387Z\"}"
        secret_key = "B00C42DAD33EAC6F6572DA756EA4915349C0A4F6"

        result = SignatureUtils.is_valid_signature(secret_key, payload, signature)
        self.assertTrue(result)

    def test_verify_request_empty_signature(self):
        result = SignatureUtils.is_valid_signature("", "", "B00C42DAD33EAC6F6572DA756EA4915349C0A4F6")
        self.assertFalse(result)



import unittest

import pytest

from securenative.utils.encryption_utils import EncryptionUtils


class EncryptionUtilsTest(unittest.TestCase):

    def setUp(self):
        self.SECRET_KEY = "B00C42DAD33EAC6F6572DA756EA4915349C0A4F6"
        self.PAYLOAD = "{\"cid\":\"198a41ff-a10f-4cda-a2f3-a9ca80c0703b\",\"vi\":\"148a42ff-b40f-4cda-a2f3-a8ca80c0703b\",\"fp\":\"6d8cabd95987f8318b1fe01593d5c2a5.24700f9f1986800ab4fcc880530dd0ed\"}"
        self.CID = "198a41ff-a10f-4cda-a2f3-a9ca80c0703b"
        self.FP = "6d8cabd95987f8318b1fe01593d5c2a5.24700f9f1986800ab4fcc880530dd0ed"

    def test_decrypt(self):
        encrypted_payload = "5208ae703cc2fa0851347f55d3b76d3fd6035ee081d71a401e8bc92ebdc25d42440f62310bda60628537744ac03f200d78da9e61f1019ce02087b7ce6c976e7b2d8ad6aa978c532cea8f3e744cc6a5cafedc4ae6cd1b08a4ef75d6e37aa3c0c76954d16d57750be2980c2c91ac7ef0bbd0722abd59bf6be22493ea9b9759c3ff4d17f17ab670b0b6fc320e6de982313f1c4e74c0897f9f5a32d58e3e53050ae8fdbebba9009d0d1250fe34dcde1ebb42acbc22834a02f53889076140f0eb8db1"
        result = EncryptionUtils.decrypt(encrypted_payload, self.SECRET_KEY)

        self.assertEqual(result.cid, self.CID)
        self.assertEqual(result.fp, self.FP)

    @pytest.mark.skip("Differences in crypto version fails this test when in reality it's passing")
    def test_encrypt(self):
        result = EncryptionUtils.encrypt(self.PAYLOAD, self.SECRET_KEY)

        self.assertIsNotNone(result)
        self.assertGreater(len(self.PAYLOAD), len(result))



import json
from binascii import unhexlify, hexlify

from Crypto import Random
from Crypto.Cipher import AES

from securenative.logger import Logger
from securenative.models.client_token import ClientToken


class EncryptionUtils(object):
    BLOCK_SIZE = 16
    KEY_SIZE = 32

    @classmethod
    def encrypt(cls, text, cipher_key):
        try:
            key = cipher_key[:cls.KEY_SIZE]
            iv = Random.new().read(AES.block_size)
            cipher = AES.new(key, AES.MODE_CBC, iv)
            raw = str(cls._pad(text))
            return hexlify(iv + cipher.encrypt(raw))
        except Exception as e:
            Logger.error("Could not encrypt text {}; {}".format(text, e))
            return None

    @classmethod
    def decrypt(cls, encrypted, cipher_key):
        try:
            key = cipher_key[:cls.KEY_SIZE]
            content = unhexlify(encrypted)
            iv = content[:cls.BLOCK_SIZE]
            cipher_text = content[cls.BLOCK_SIZE:]
            aes = AES.new(key, AES.MODE_CBC, iv)
            rv = aes.decrypt(cipher_text).decode("utf-8").strip()
            secret = json.loads(rv)
            return ClientToken(secret.get("cid"), secret.get("vid"), secret.get("fp"))
        except Exception as e:
            Logger.error("Could not decrypt str {}; {}".format(encrypted, e))
            return None

    @classmethod
    def _pad(cls, s):
        return lambda s: s + (cls.BLOCK_SIZE - len(s) % cls.BLOCK_SIZE) * chr(cls.BLOCK_SIZE - len(s) % cls.BLOCK_SIZE)




from securenative.enums.failover_strategy import FailOverStrategy


class SecureNativeOptions(object):

    def __init__(self, api_key=None, api_url="https://api.securenative.com/collector/api/v1", interval=1000,
                 max_events=1000, timeout=1500, auto_send=True, disable=False, log_level="CRITICAL",
                 fail_over_strategy=FailOverStrategy.FAIL_OPEN.value):
        self.api_key = api_key
        self.api_url = api_url
        self.interval = interval
        self.max_events = max_events
        self.timeout = timeout
        self.auto_send = auto_send
        self.disable = disable
        self.log_level = log_level
        self.fail_over_strategy = fail_over_strategy



import uuid

from securenative.context.context_builder import ContextBuilder
from securenative.models.request_context import RequestContextBuilder
from securenative.utils.date_utils import DateUtils
from securenative.utils.encryption_utils import EncryptionUtils


class SDKEvent(object):

    def __init__(self, event_options, securenative_options):
        if event_options.context is not None:
            self.context = event_options.context
        else:
            self.context = ContextBuilder.default_context_builder().build()

        client_token = EncryptionUtils.decrypt(self.context.client_token, securenative_options.api_key)

        self.rid = str(uuid.uuid4())
        self.event_type = event_options.event.value
        self.user_id = event_options.user_id
        self.user_traits = event_options.user_traits
        self.request = RequestContextBuilder() \
            .with_cid(client_token.cid if client_token else "") \
            .with_vid(client_token.vid if client_token else "") \
            .with_fp(client_token.fp if client_token else "") \
            .with_ip(self.context.ip) \
            .with_remote_ip(self.context.remote_ip) \
            .with_method(self.context.method) \
            .with_url(self.context.url) \
            .with_headers(self.context.headers) \
            .build()

        self.timestamp = DateUtils.to_timestamp(event_options.timestamp)
        self.properties = event_options.properties



import unittest

import responses

from securenative.config.configuration_manager import ConfigurationManager
from securenative.http.securenative_http_client import SecureNativeHttpClient


class SecureNativeHttpClientTest(unittest.TestCase):

    @responses.activate
    def test_should_make_simple_post_call(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY").\
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/track",
                      json={"event": "SOME_EVENT_NAME"}, status=200)
        client = SecureNativeHttpClient(options)
        payload = "{\"event\": \"SOME_EVENT_NAME\"}"

        res = client.post("track", payload)

        self.assertEqual(res.ok, True)
        self.assertEqual(res.status_code, 200)
        self.assertEqual(res.text, payload)



class RequestUtils(object):
    SECURENATIVE_COOKIE = "_sn"
    SECURENATIVE_HEADER = "x-securenative"

    @staticmethod
    def get_secure_header_from_request(headers):
        if headers:
            return headers[RequestUtils.SECURENATIVE_HEADER]
        return []

    @staticmethod
    def get_client_ip_from_request(request):
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[-1].strip()
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip

    @staticmethod
    def get_remote_ip_from_request(request):
        try:
            return request.raw._original_response.fp.raw._sock.getpeername()[0]
        except AttributeError:
            return ""



import os
from configparser import ConfigParser, NoSectionError

from securenative.config.configuration_builder import ConfigurationBuilder


class ConfigurationManager(object):
    DEFAULT_CONFIG_FILE = "securenative.ini"
    CUSTOM_CONFIG_FILE_ENV_NAME = "SECURENATIVE_COMFIG_FILE"
    config = ConfigParser()

    @classmethod
    def read_resource_file(cls, resource_path):
        cls.config.read(resource_path)

        properties = {}
        for key, value in cls.config.defaults().items():
            properties[key.upper()] = value

        return properties

    @classmethod
    def _get_resource_path(cls, env_name):
        env_value = os.environ.get(env_name)

        if env_value:
            return env_value

        return os.environ.get(cls.DEFAULT_CONFIG_FILE)

    @staticmethod
    def config_builder():
        return ConfigurationBuilder.default_config_builder()

    @classmethod
    def _get_env_or_default(cls, properties, key, default):
        if os.environ.get(key):
            return os.environ.get(key)
        if properties.get(key):
            return properties.get(key)
        return default

    @classmethod
    def load_config(cls):
        options = ConfigurationBuilder().get_default_securenative_options()

        resource_path = cls.DEFAULT_CONFIG_FILE
        if os.environ.get(cls.CUSTOM_CONFIG_FILE_ENV_NAME):
            resource_path = os.environ.get(cls.CUSTOM_CONFIG_FILE_ENV_NAME)

        properties = cls.read_resource_file(resource_path)

        return ConfigurationBuilder(). \
            with_api_key(cls._get_env_or_default(properties, "SECURENATIVE_API_KEY", options.api_key)). \
            with_api_url(cls._get_env_or_default(properties, "SECURENATIVE_API_URL", options.api_url)). \
            with_interval(cls._get_env_or_default(properties, "SECURENATIVE_INTERVAL", options.interval)). \
            with_max_events(cls._get_env_or_default(properties, "SECURENATIVE_MAX_EVENTS", options.max_events)). \
            with_timeout(cls._get_env_or_default(properties, "SECURENATIVE_TIMEOUT", options.timeout)). \
            with_auto_send(cls._get_env_or_default(properties, "SECURENATIVE_AUTO_SEND", options.auto_send)). \
            with_disable(cls._get_env_or_default(properties, "SECURENATIVE_DISABLE", options.disable)). \
            with_log_level(cls._get_env_or_default(properties, "SECURENATIVE_LOG_LEVEL", options.log_level)). \
            with_fail_over_strategy(cls._get_env_or_default(
                properties, "SECURENATIVE_FAILOVER_STRATEGY", options.fail_over_strategy))



import setuptools
from setuptools import setup


with open("README.md", "r") as fh:
    long_description = fh.read()

with open('VERSION') as f:
    version = f.read()

setup(
    name='securenative',
    packages=setuptools.find_packages(),
    version=version,
    license='MIT',
    description='Secure Native SDK for python',
    author='SecureNative',
    author_email='support@securenative.com',
    url='http://www.securenative.com',
    download_url='https://github.com/securenative/securenative-python/archive/0.1.tar.gz',
    keywords=["securenative", 'cyber-security'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "requests",
        "pycrypto",
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



import unittest

import requests_mock

from securenative.context.context_builder import ContextBuilder


class ContextBuilderTest(unittest.TestCase):

    def test_create_context_from_request(self):
        with requests_mock.Mocker(real_http=True) as request:
            request.server_name = "www.securenative.com"
            request.request_uri = "/login"
            request.query_string = "param1=value1&param2=value2"
            request.method = "Post"
            request.url = "www.securenative.com"
            request.request_uri = "/login/param1=value1&param2=value2"
            request.META = {
                "REMOTE_ADDR": "51.68.201.122"
            }
            request.headers = {
                "x-securenative": "71532c1fad2c7f56118f7969e401f3cf080239140d208e7934e6a530818c37e544a0c2330a487bcc6fe4f662a57f265a3ed9f37871e80529128a5e4f2ca02db0fb975ded401398f698f19bb0cafd68a239c6caff99f6f105286ab695eaf3477365bdef524f5d70d9be1d1d474506b433aed05d7ed9a435eeca357de57817b37c638b6bb417ffb101eaf856987615a77a"}

            context = ContextBuilder.from_http_request(request).build()

            self.assertEqual(context.client_token,
                             "71532c1fad2c7f56118f7969e401f3cf080239140d208e7934e6a530818c37e544a0c2330a487bcc6fe4f662a57f265a3ed9f37871e80529128a5e4f2ca02db0fb975ded401398f698f19bb0cafd68a239c6caff99f6f105286ab695eaf3477365bdef524f5d70d9be1d1d474506b433aed05d7ed9a435eeca357de57817b37c638b6bb417ffb101eaf856987615a77a")
            self.assertEqual(context.ip, "51.68.201.122")
            self.assertEqual(context.method, "Post")
            self.assertEqual(context.url, "www.securenative.com")
            self.assertEqual(context.remote_ip, "")
            self.assertEqual(context.headers, {
                "x-securenative": "71532c1fad2c7f56118f7969e401f3cf080239140d208e7934e6a530818c37e544a0c2330a487bcc6fe4f662a57f265a3ed9f37871e80529128a5e4f2ca02db0fb975ded401398f698f19bb0cafd68a239c6caff99f6f105286ab695eaf3477365bdef524f5d70d9be1d1d474506b433aed05d7ed9a435eeca357de57817b37c638b6bb417ffb101eaf856987615a77a"})
            self.assertIsNone(context.body)

    def test_create_context_from_request_with_cookie(self):
        with requests_mock.Mocker(real_http=True) as request:
            request.server_name = "www.securenative.com"
            request.request_uri = "/login"
            request.query_string = "param1=value1&param2=value2"
            request.method = "Post"
            request.url = "www.securenative.com"
            request.request_uri = "/login/param1=value1&param2=value2"
            request.META = {
                "REMOTE_ADDR": "51.68.201.122"
            }
            request.cookies = {"_sn":
                                     "71532c1fad2c7f56118f7969e401f3cf080239140d208e7934e6a530818c37e544a0c2330a487bcc6fe4f662a57f265a3ed9f37871e80529128a5e4f2ca02db0fb975ded401398f698f19bb0cafd68a239c6caff99f6f105286ab695eaf3477365bdef524f5d70d9be1d1d474506b433aed05d7ed9a435eeca357de57817b37c638b6bb417ffb101eaf856987615a77a"}

            context = ContextBuilder.from_http_request(request).build()

            self.assertEqual(context.client_token,
                             "71532c1fad2c7f56118f7969e401f3cf080239140d208e7934e6a530818c37e544a0c2330a487bcc6fe4f662a57f265a3ed9f37871e80529128a5e4f2ca02db0fb975ded401398f698f19bb0cafd68a239c6caff99f6f105286ab695eaf3477365bdef524f5d70d9be1d1d474506b433aed05d7ed9a435eeca357de57817b37c638b6bb417ffb101eaf856987615a77a")
            self.assertEqual(context.ip, "51.68.201.122")
            self.assertEqual(context.method, "Post")
            self.assertEqual(context.url, "www.securenative.com")
            self.assertEqual(context.remote_ip, "")
            self.assertIsNone(context.body)

    def test_create_default_context_builder(self):
        context = ContextBuilder.default_context_builder().build()

        self.assertIsNone(context.client_token)
        self.assertIsNone(context.ip)
        self.assertIsNone(context.method)
        self.assertIsNone(context.url)
        self.assertIsNone(context.remote_ip)
        self.assertIsNone(context.headers)
        self.assertIsNone(context.body)

    def test_create_custom_context_with_context_builder(self):
        context = ContextBuilder.default_context_builder(). \
            with_url("/some-url"). \
            with_client_token("SECRET_TOKEN"). \
            with_ip("10.0.0.0"). \
            with_body("{ \"name\": \"YOUR_NAME\" }"). \
            with_method("Get"). \
            with_remote_ip("10.0.0.1"). \
            with_headers({"header1": "value1"}). \
            build()

        self.assertEqual(context.url, "/some-url")
        self.assertEqual(context.client_token, "SECRET_TOKEN")
        self.assertEqual(context.ip, "10.0.0.0")
        self.assertEqual(context.body, "{ \"name\": \"YOUR_NAME\" }")
        self.assertEqual(context.method, "Get")
        self.assertEqual(context.remote_ip, "10.0.0.1")
        self.assertEqual(context.headers, {"header1": "value1"})



import unittest

from securenative.config.configuration_manager import ConfigurationManager
from securenative.enums.failover_strategy import FailOverStrategy
from securenative.exceptions.securenative_config_exception import SecureNativeConfigException
from securenative.exceptions.securenative_sdk_Illegal_state_exception import SecureNativeSDKIllegalStateException
from securenative.exceptions.securenative_sdk_exception import SecureNativeSDKException
from securenative.securenative import SecureNative


class SecureNativeTest(unittest.TestCase):

    def test_get_sdk_instance_without_init_throws(self):
        with self.assertRaises(SecureNativeSDKIllegalStateException):
            SecureNative.get_instance()

    def test_init_sdk_without_api_key_should_throw(self):
        with self.assertRaises(SecureNativeSDKException):
            SecureNative.init_with_options(ConfigurationManager.config_builder())

    def test_init_sdk_with_empty_api_key_should_throw(self):
        with self.assertRaises(SecureNativeConfigException):
            SecureNativeSDKException, SecureNative.init_with_api_key("")

    def test_init_sdk_with_api_key_and_defaults(self):
        SecureNative._flush()
        api_key = "API_KEY"
        securenative = SecureNative.init_with_api_key(api_key)
        options = securenative.get_options()

        self.assertEqual(options.api_key, api_key)
        self.assertEqual(options.api_url, "https://api.securenative.com/collector/api/v1")
        self.assertEqual(options.interval, 1000)
        self.assertEqual(options.timeout, 1500)
        self.assertEqual(options.max_events, 1000)
        self.assertEqual(options.auto_send, True)
        self.assertEqual(options.disable, False)
        self.assertEqual(options.log_level, "CRITICAL")
        self.assertEqual(options.fail_over_strategy, FailOverStrategy.FAIL_OPEN.value)

    def test_init_sdk_twice_will_throw(self):
        with self.assertRaises(SecureNativeSDKException):
            SecureNative.init_with_api_key("API_KEY")
            SecureNative.init_with_api_key("API_KEY")

    def test_init_sdk_with_api_key_and_get_instance(self):
        SecureNative._flush()
        api_key = "API_KEY"
        securenative = SecureNative.init_with_api_key(api_key)

        self.assertEqual(securenative, SecureNative.get_instance())

    def test_init_sdk_with_builder(self):
        SecureNative._flush()
        securenative = SecureNative.init_with_options(SecureNative.config_builder().
                                                      with_api_key("API_KEY").
                                                      with_max_events(10).
                                                      with_log_level("ERROR"))

        options = securenative.get_options()
        self.assertEqual(options.api_key, "API_KEY")
        self.assertEqual(options.max_events, 10)
        self.assertEqual(options.log_level, "ERROR")



import requests

from securenative.utils.version_utils import VersionUtils


class SecureNativeHttpClient(object):
    AUTHORIZATION_HEADER = "Authorization"
    VERSION_HEADER = "SN-Version"
    USER_AGENT_HEADER = "User-Agent"
    USER_AGENT_HEADER_VALUE = "SecureNative-python"
    CONTENT_TYPE_HEADER = "Content-Type"
    CONTENT_TYPE_HEADER_VALUE = "application/json"

    def __init__(self, securenative_options):
        self.options = securenative_options

    def _headers(self):
        return {
            self.CONTENT_TYPE_HEADER: self.CONTENT_TYPE_HEADER_VALUE,
            self.USER_AGENT_HEADER: self.USER_AGENT_HEADER_VALUE,
            self.VERSION_HEADER: VersionUtils.get_version(),
            self.AUTHORIZATION_HEADER: self.options.api_key
        }

    def post(self, path, body):
        url = "{}/{}".format(self.options.api_url, path)
        return requests.post(url=url, data=body, headers=self._headers())



import json
import unittest
from datetime import datetime

import responses

from securenative.config.configuration_manager import ConfigurationManager
from securenative.event_manager import EventManager
from securenative.models.request_context import RequestContext
from securenative.models.user_traits import UserTraits


class SampleEvent(object):

    def __init__(self):
        self.event_type = "custom-event"
        self.timestamp = datetime.now().strftime("%Y-%d-%dT%H:%M:%S.%fZ")
        self.rid = "432532"
        self.user_id = "1"
        self.user_traits = UserTraits("some user", "email@securenative.com")
        self.request = RequestContext()
        self.properties = []


class EventManagerTest(unittest.TestCase):

    def setUp(self):
        self.event = SampleEvent()

    @responses.activate
    def test_should_successfully_send_sync_event_with_status_code_200(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY"). \
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        res_body = "{\"data\": true}"
        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/some-path/to-api",
                      json=json.loads(res_body), status=200)
        event_manager = EventManager(options)

        data = event_manager.send_sync(self.event, "some-path/to-api", False)
        self.assertEqual(res_body, data.text)

    @responses.activate
    def test_should_send_sync_event_and_fail_when_status_code_401(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY"). \
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/some-path/to-api",
                      json={}, status=401)
        event_manager = EventManager(options)

        res = event_manager.send_sync(self.event, "some-path/to-api", False)

        self.assertEqual(res.status_code, 401)

    @responses.activate
    def test_should_send_sync_event_and_fail_when_status_code_500(self):
        options = ConfigurationManager.config_builder(). \
            with_api_key("YOUR_API_KEY"). \
            with_api_url("https://api.securenative-stg.com/collector/api/v1")

        responses.add(responses.POST, "https://api.securenative-stg.com/collector/api/v1/some-path/to-api",
                      json={}, status=500)
        event_manager = EventManager(options)

        res = event_manager.send_sync(self.event, "some-path/to-api", False)

        self.assertEqual(res.status_code, 500)
